/* $Id: midrstr.c,v 1.5 2002/01/19 19:10:22 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            midrstr.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Machine Independent Data Representation, routines
 *                  for handling character strings and byte blocks.
 *
 *                  See midrgen.c for more info.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <string.h>

#include "shhmidr.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          midrPutStr
 *
 *  FUNCTION      Enter a string in the buffer of the MIDR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                void midrPutStr(MIDR *midr, const char *s);
 *
 *  INPUT         midr    the structure in question.
 *                s       the string to store.
 *
 *  DESCRIPTION   This function handles C strings, that is; '\0'-terminated
 *                strings. No padding to a certain size is done, so the
 *                buffer will be increased by the length of the string and
 *                it's '\0'-terminator, whatever that length might be.
 */
void
midrPutStr(MIDR *midr, const char *s)
{
    char *p;

    p = (char *) (midr->buff + midr->idx);
    strcpy(p, s);
    if ((midr->idx += strlen(s) + 1) > midr->len)
	midr->len = midr->idx;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrGetStr
 *
 *  FUNCTION      Fetch a string from the MIDR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                char *midrGetString(MIDR *midr, char *s, size_t size);
 *
 *  INPUT         midr    the structure in question.
 *                size    the size of the buffer.
 *
 *  OUTPUT        s       The buffer that will contain the string.
 *
 *  RETURNS       s
 *
 *  DESCRIPTION   This function handles '\0'-terminated strings.
 */
char *
midrGetStr(MIDR *midr, char *s, size_t size)
{
    int  len;
    char *p;

    p = (char *) (midr->buff + midr->idx);
    len = strlen(p);
    strncpy(s, p, size - 1);
    s[size - 1] = '\0';  /* In case strncpy didn't copy the last byte */
    midr->idx += len + 1;
    return s;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrPutByteBlock
 *
 *  FUNCTION      Enter a block of bytes in the buffer of the MIDR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                void midrPutByteBlock(MIDR *midr, const char *b,
 *                                      size_t size);
 *
 *  INPUT         midr    the structure in question.
 *                b       the byte block to store.
 *                size    the size of the buffer.
 */
void
midrPutByteBlock(const BYTE *b, size_t size, MIDR *midr)
{
    memcpy(midr->buff + midr->size, b, size);
    if ((midr->idx += size) > midr->len)
	midr->len = midr->idx;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          midrGetByteBlock
 *
 *  FUNCTION      Fetch a block of bytes from the MIDR-structure.
 *
 *  SYNOPSIS      #include "shhmidr.h"
 *                char *midrGetByteBlocking(MIDR *midr, char *b, size_t size);
 *
 *  INPUT         midr    the structure in question.
 *                size    the size of the buffer, and also the max length
 *                        of the string, including terminating '\0'.
 *
 *  OUTPUT        b       The buffer that will contain the byte block.
 *
 *  RETURNS       b
 */
BYTE *
midrGetByteBlock(MIDR *midr, BYTE *b, size_t size)
{
    memcpy(b, midr->buff + midr->idx, size);
    midr->idx += size;
    return b;
}
