/* $Id: multicast.c,v 1.5 2002/01/19 19:10:22 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            multicast.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Multicast functions.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          udpMulticastOpen
 *
 *  FUNCTION      Join a multicast group for sending and receiving.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr updMulticastOpen(const char *group, const char *port,
 *                                       int ttl, int *sock);
 *
 *  INPUT         group   group as a name or dotted quad.
 *                port    port number (or service name).
 *                ttl     time to live value for sending.
 *
 *  OUTPUT        sock    the socket.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   Sets up necessary stuff to join a multicast group,
 *                with the given TTL for outgoing packets.
 */
SNErr
udpMulticastOpen(const char *group, const char *port, int ttl, int *sock)
{
    SNAddr sna;
    SNPort snp;
    SNErr  err;

    /* get the port number corresponding to the given service. */
    if ((err = inServiceToPort(port, "udp", &snp)) != SN_OK)
	return err;

    /* parse the server host address. */
    if ((err = inHostNameToAddr(group, &sna)) != SN_OK)
	return err;

    /* create a socket. */
    if ((err = sockCreate(SN_PROTO_UDP, sock)) != SN_OK)
	return err;

    /* make sure several may bind the socket, then bind it. */
    sockSetReuseAddr(*sock, 1);
    if ((err = sockBind(*sock, SN_ANY_ADDR, snp)) != SN_OK)
	return err;

    /* set the wanted time to live value. */
    sockSetMulticastTtl(*sock, ttl);

    /* join the multicast group */
    return sockJoinMulticastGroup(*sock, sna);
}
