/* $Id: select.c,v 1.4 2002/01/19 19:10:22 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            select.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <errno.h>

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          snSelect
 *
 *  FUNCTION      select(2)-wrapper.
 *
 *  SYNOPSIS      #include <shhnet.h>
 *                SNErr snSelect(int *n, fd_set *readfds, fd_set *writefds,
 *                               fd_set *exceptfds, struct timeval *timeout);
 *
 *  INPUT         n       the highest numbered descriptor in any of the
 *                        three following sets. a good value for this, is
 *                        the constant FD_SETSIZE.
 *                readfds set of file descriptors to be monitored for
 *                        readability. may be NULL if the set is empty.
 *                        use FD_ZERO and FD_SET to initiate the set
 *                        (see man select).
 *                writefds
 *                        set of descriptors to check for writability.
 *                        may be NULL.
 *                exceotfds
 *                        set of descriptors to watch for exceptions.
 *                        may be NULL.
 *                timeout an upper bound on the time elapsed before the
 *                        function returns. NULL indicates no timeout.
 *
 *  OUTPUT        n       number of `ready' file descriptors in the sets.
 *                readfds set of file descriptors ready for reading.
 *                        use FD_ISSET to test which descriptors are
 *                        available (see man select).
 *                writefds
 *                        set of descriptors ready for writing.
 *                exceotfds
 *                        set of descriptors with incoming exception.
 *                timeout time left of initial timeout value.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   The function returns either when any of the indicated
 *                selectors are ready for the indicated action, or when
 *                the time in timout has expired.
 *                Works like the normal select, except from the following:
 *                  * it will correctly handle EINTR-errors by checking how
 *                    much of the timeout is left, instead of starting all
 *                    over again.
 *                  * when a file descriptor status change is returned, the
 *                    timeout value is updated to contain the time left
 *                    to wait for a full timeout.
 */
SNErr
snSelect(int *n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds,
	 struct timeval *timeout)
{
    int    ret;
    struct timeval tv, before, after, *tvp;

    do {
	if (timeout) {
	    gettimeofday(&before, NULL);
	    tv = *timeout;
	    tvp = &tv;
	} else
	    tvp = NULL;
        ret = select(*n, readfds, writefds, exceptfds, tvp);
	if (timeout) {
	    if (ret == 0) {
		timeout->tv_sec = timeout->tv_usec = 0L;
	    } else {
		gettimeofday(&after, NULL);
		after.tv_sec -= before.tv_sec;
		if ((after.tv_usec -= before.tv_usec) < 0L) {
		    after.tv_usec += 1000000L;
		    --after.tv_sec;
		}
		timeout->tv_sec -= after.tv_sec;
		if ((timeout->tv_usec -= after.tv_usec) < 0L) {
		    timeout->tv_usec += 1000000L;
		    if (--timeout->tv_sec < 0L)
			timeout->tv_sec = timeout->tv_usec = 0L;
		}
	    }
	}
    } while (ret < 0 && errno == EINTR);
    if (ret < 0)
	return _snError(SN_ERR_ERRNO, "select");
    *n = ret;
    return _snError(SN_OK, NULL);
}
