/* $Id: tcpio.c,v 0.12 2002/01/19 19:10:22 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            tcpio.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Functions for doing IO on a tcp-connection. The
 *                  functions handle possible interruptions in the middle
 *                  of packets.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          tcpRead
 *
 *  FUNCTION      Read from a tcp-connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr tcpRead(int sock, char *buf,
 *                              size_t count, size_t *nread);
 *
 *  INPUT         sock    descriptor of tcp-stream.
 *                buf     pointer to buffer that will receive the data.
 *                count   number of bytes to read.
 *
 *  OUTPUT        nread   number of bytes read.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   Functions like a normal read(); since reading is done
 *                from a network connection, the number of bytes read may
 *                be less than the number of bytes wanted, and more may
 *                be available later. Use tcpReadAll() if you want to read
 *                all count bytes.
 */
SNErr
tcpRead(int sock, char *buf, size_t count, size_t *nread)
{
    long n;

    if ((n = read(sock, buf, count)) < 0)
	return _snError(SN_ERR_ERRNO, "read");
    *nread = n;
    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          tcpWrite
 *
 *  FUNCTION      Write to a tcp-connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr tcpWrite(int sock, char *buf,
 *                               size_t count, size_t *nwritten);
 *
 *  INPUT         sock    descriptor of tcp-stream.
 *                buf     pointer to buffer that contains the data.
 *                count   number of bytes to write.
 *
 *  OUTPUT        nwritten
 *                        number of bytes written.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   Functions like a normal write(); since writing is done
 *                to a network connection, the number of bytes written may
 *                be less than the number of bytes intended. Use
 *                tcpWriteAll() if you want to write all count bytes.
 */
SNErr
tcpWrite(int sock, const char *buf, size_t count, size_t *nwritten)
{
    long n;

    if ((n = write(sock, buf, count)) < 0)
	return _snError(SN_ERR_ERRNO, "write");
    *nwritten = n;
    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          tcpReadAll
 *
 *  FUNCTION      Read from a tcp-connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr tcpReadAll(int sock, char *buf, size_t count);
 *
 *  INPUT         sock    descriptor of tcp-stream.
 *                buf     pointer to buffer that will receive the data.
 *                count   number of bytes to read.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   Functions like a normal read(), but doesn't return untill
 *                all count bytes are read, or an error occurs.
 */
SNErr
tcpReadAll(int sock, char *buf, size_t count)
{
    int nleft, nread;

    nleft = count;
    do {
	if ((nread = read(sock, buf, nleft)) < 0)
	    return _snError(SN_ERR_ERRNO, "read");
	else if (nread == 0)
	    break;
	nleft -= nread;
	buf += nread;
    } while (nleft);
    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          tcpWriteAll
 *
 *  FUNCTION      Write to a tcp-connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr tcpWriteAll(int sock, char *buf, size_t count);
 *
 *  INPUT         sock    descriptor of tcp-stream.
 *                buf     pointer to buffer that contains the data.
 *                count   number of bytes to write.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   Functions like a normal write(), but doesn't return untill
 *                all count bytes are written, or an error occurs.
 */
SNErr
tcpWriteAll(int sock, const char *buf, size_t count)
{
    int nleft, nwritten;

    nleft = count;
    do {
	if ((nwritten = write(sock, buf, nleft)) < 0)
	    return _snError(SN_ERR_ERRNO, "write");
	else if (nwritten == 0)
	    break;
	nleft -= nwritten;
	buf += nwritten;
    } while (nleft);
    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          tcpClose
 *
 *  FUNCTION      Close a tcp-connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr tcpClose(int sock);
 *
 *  INPUT         sock    descriptor of tcp-connection.
 *
 *  RETURNS       Error code.
 */
SNErr
tcpClose(int sock)
{
    if (close(sock) < 0)
	return _snError(SN_ERR_ERRNO, "close");
    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          tcpGets
 *
 *  FUNCTION      Read a string (line) from the given tcp-socket.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr tcpGets(int sock, char *s, int n);
 *
 *  INPUT         sock    descriptor of tcp-connection.
 *                n       read no more than n-1 characters.
 *
 *  OUTPUT        s       the buffer to fill with the string.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   Stops reading when n-1 characters or a '\n' is read,
 *                or when no more data is available.
 *                Any '\n' is stored in the buffer, and a '\0' is added
 *                at the end.
 *                A string of length 0 indicates that the other end has
 *                closed the connection.
 */
SNErr
tcpGets(int sock, char *s, int n)
{
    int nc, l = 0;

    for (;;) {
	if (l == n - 1)
	    break;
	if ((nc = read(sock, s, 1)) < 0)
	    return _snError(SN_ERR_ERRNO, "read");
	if (nc == 0)
	    break;
	++l;
	++s;
	if (*(s - 1) == '\n')
	    break;
    }
    *s = '\0';
    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          tcpPuts
 *
 *  FUNCTION      Send a string to the given tcp-socket.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr tcpPuts(int sock, const char *s);
 *
 *  INPUT         sock    descriptor of tcp-connection.
 *                s       the string to send.
 *
 *  RETURNS       Whatever tcpWriteAll() returns.
 *
 *  DESCRIPTION   Sends the string without the trailing '\0'.
 */
SNErr
tcpPuts(int sock, const char *s)
{
    return tcpWriteAll(sock, s, strlen(s));
}



/*-------------------------------------------------------------------------
 *
 *  NAME          tcpPrintf
 *
 *  FUNCTION      Send given message to the tcp-socket.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr tcpPrintf(int sock, const char *format, ...);
 *
 *  INPUT         sock    descriptor of tcp-connection.
 *                format, ...
 *                        arguments used as with printf().
 *
 *                        NOTE! The expanded string must not be longer
 *                              than 400 characters!
 *
 *  RETURNS       Whatever tcpWriteAll() returns.
 */
SNErr
tcpPrintf(int sock, const char *format, ...)
{
    va_list ap;
    char    buffer[400];

    va_start(ap, format);
    vsprintf(buffer, format, ap);
    va_end(ap);
    return tcpWriteAll(sock, buffer, strlen(buffer));
}
