/* $Id: tcpserv.c,v 0.11 2002/01/19 19:10:22 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            tcpserv.c
 *  MODULE OF       shhnet - TCP/UDP networking library.
 *
 *  DESCRIPTION     Functions specific to the server side of
 *                  a TCP connection.
 *
 *  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 *
 **************************************************************************/

#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include "internal.h"
#include "shhnet.h"

/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

/*-------------------------------------------------------------------------
 *
 *  NAME          tcpServerOpen
 *
 *  FUNCTION      Open a server side TCP connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr tcpServerOpen(const char *service, int *sock);
 *
 *  INPUT         addr    local address to bind to.
 *                service a service name, or a port number (ascii). if a
 *                        service name is given, it is mapped to a port
 *                        number.
 *
 *  OUTPUT        sock    the socket referencing this side of the
 *                        connection.
 *
 *  DESCRIPTION   See _snSrvOpenCommon() in servopen.c
 */
SNErr
tcpServerOpenAddr(SNAddr addr, const char *service, int *sock)
{
    return _snSrvOpenAddrCommon(SN_PROTO_TCP, addr, service, sock);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          tcpServerOpen
 *
 *  FUNCTION      Open a server side TCP connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                SNErr tcpServerOpen(const char *service, int *sock);
 *
 *  INPUT         service a service name, or a port number (ascii). if a
 *                        service name is given, it is mapped to a port
 *                        number.
 *
 *  OUTPUT        sock    the socket referencing this side of the
 *                        connection.
 *
 *  DESCRIPTION   See _snSrvOpenCommon() in servopen.c
 */
SNErr
tcpServerOpen(const char *service, int *sock)
{
    return _snSrvOpenCommon(SN_PROTO_TCP, service, sock);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          tcpListen
 *
 *  FUNCTION      Start listening for incoming connections.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                int tcpListen(int sock, int backlog);
 *
 *  INPUT         sock    socket of TCP connection.
 *                backlog max number of new callers to put in a wait queue
 *                        while serving others.
 *
 *  RETURNS       Error code.
 *
 *  DESCRIPTION   Many Unix implementations silently limits the backlog
 *                to 5 callers.
 */
SNErr
tcpListen(int sock, int backlog)
{
    if (listen(sock, backlog) < 0)
	return _snError(SN_ERR_ERRNO, "listen");
    return _snError(SN_OK, NULL);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          tcpAccept
 *
 *  FUNCTION      Accept an incoming TCP connection.
 *
 *  SYNOPSIS      #include "shhnet.h"
 *                int tcpAccept(int sock, int *newsock);
 *
 *  INPUT         sock    socket of TCP connection.
 *
 *  OUTPUT        newsock the new socket.
 *
 *  RETURNS       Error code.
 */
SNErr
tcpAccept(int sock, int *newsock)
{
    int    clilen;
    struct sockaddr_in cli;

    clilen = sizeof(cli);
    /* accept is one of the system calls on BSD4.3 that are not
     * automatically restarted when interrupted by signals, so we wrap
     * it in a little loop.  */
    do {
	*newsock = accept(sock, (struct sockaddr *) &cli, &clilen);
    } while (*newsock < 0 && errno == EINTR);
    if (*newsock < 0)
	return _snError(SN_ERR_ERRNO, "accept");
    return _snError(SN_OK, NULL);
}
